﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
//using System.Threading.Tasks;
using UnityEngine;

public class Taudrive : PartModule
{
    [KSPField]
    public float fuelefficiency = 1F;
    [KSPField]
    public float particlegrate = 800F;
    [KSPField]
    public float ConvertRatio = 1F;
    public Vector4 color = Vector4.zero;


    [KSPField(isPersistant = true)]
    public bool engineIgnited = false;
    public bool flameOut = false;
    public bool agActivated = false;
    public bool depleted = false;
    public bool ecActivated = false;


    //    public bool staged = false;
    private ParticleEmitter emt;
    public float particleSize = 0.001f;

    private float rotation = 0F;

    private GameObject rotor;
    private GameObject stator;

    [KSPField(guiName = "Engine Status", guiActive = true)]
    private string ES = "Deactivated";

    [KSPField(guiName = "Mass", guiActive = true)]
    private string mass = "N/a";

    [KSPField(guiActive = true, guiActiveEditor = true, guiName = "Max Overload", isPersistant = true), UI_FloatRange(minValue = 0f, maxValue = 5f, stepIncrement = 0.1f)]
    public float Overload = 1f;

    [KSPAction("Toggle", KSPActionGroup.None, guiName = "Toggle Engine")]
    private void ActionActivate(KSPActionParam param)
    {
        if (engineIgnited == true)
        {
            Deactivate();
        }
        else
        {
            Activate();
        }
    }

    [KSPEvent(name = "Activate", guiName = "Activate Engine", active = true, guiActive = true)]
    public void Activate()
    {
        if (depleted == false)
        {
            engineIgnited = true;
            Events["Deactivate"].guiActive = true;
            Events["Activate"].guiActive = false;
        }
    }

    [KSPEvent(name = "Deactivate", guiName = "Deactivate Engine", active = true, guiActive = false)]
    public void Deactivate()
    {
        engineIgnited = false;
        Events["Deactivate"].guiActive = false;
        Events["Activate"].guiActive = true;
    }

    [KSPAction("Toggleag", KSPActionGroup.None, guiName = "Toggle Antigravity")]
    private void Toggleag(KSPActionParam param)
    {
        if (agActivated == true)
        {
            Deactivateag();
        }
        else
        {
            Activateag();
        }

    }

    [KSPEvent(name = "Activateag", guiName = "Activate Antigravity", active = true, guiActive = true)]
    public void Activateag()
    {
        if (depleted == false)
        {
            agActivated = true;
            Events["Deactivateag"].guiActive = true;
            Events["Activateag"].guiActive = false;
        }

    }

    [KSPEvent(name = "Deactivateag", guiName = "Deactivate Antigravity", active = true, guiActive = false)]
    public void Deactivateag()
    {
        agActivated = false;
        Events["Deactivateag"].guiActive = false;
        Events["Activateag"].guiActive = true;
    }

    [KSPEvent(name = "Activateec", guiName = "Activate Converter", active = true, guiActive = true)]
    public void Activateec()
    {
            ecActivated = true;
            Events["Deactivateec"].guiActive = true;
            Events["Activateec"].guiActive = false;
    }

    [KSPEvent(name = "Deactivateec", guiName = "Deactivate Converter", active = true, guiActive = false)]
    public void Deactivateec()
    {
        ecActivated = false;
        Events["Deactivateec"].guiActive = false;
        Events["Activateec"].guiActive = true;
    }

    protected Transform rotorTransform = null;

    public override void OnStart(PartModule.StartState state)
    {
        part.stagingIcon = "LIQUID_ENGINE";
        base.OnStart(state);
        {
            if (state != StartState.Editor && state != StartState.None)
            {
                this.enabled = true;
                this.part.force_activate();
            }
            else
            {
                this.enabled = false;
            }
            if (base.part.FindModelTransform("rotor").gameObject != null)
            {
                stator = base.part.FindModelTransform("stator").gameObject;
                rotor = base.part.FindModelTransform("rotor").gameObject;
            }

            if (particleSize <= 0.1F)
                particleSize = 0.5F;

            GameObject o = Instantiate(
                UnityEngine.Resources.Load("Effects/fx_exhaustFlame_blue")) as GameObject;
            o.transform.parent = stator.transform;
            o.transform.localScale = Vector3.one * 50;
            o.transform.localEulerAngles = Vector3.zero;
            o.transform.localPosition = Vector3.zero;

            emt = o.particleEmitter;
            emt.emit = true;
            emt.useWorldSpace = false;
            emt.minEnergy = 3F;
            emt.maxEnergy = 5F;
            emt.minSize = 0.06F * particleSize;
            emt.maxSize = 0.1F * particleSize;
            emt.minEmission = 0;
            emt.maxEmission = emt.minEmission;
            emt.rndVelocity = Vector3.one * particleSize;
            emt.localVelocity = Vector3.back*0.06f;
            emt.rndVelocity = Vector3.right * 1f + Vector3.up * 1f;
            (o.GetComponent<ParticleAnimator>() as ParticleAnimator).sizeGrow = 0;

        }
    }


    public override void OnFixedUpdate()
    {
        ES = "Deactivated";
        if (depleted == true && part.Resources["GNparticle"].amount == part.Resources["GNparticle"].maxAmount)
        {
            depleted = false;
        }
        if (!HighLogic.LoadedSceneIsFlight || !vessel.isActiveVessel) return;
        float pitch = vessel.ctrlState.pitch;
        float roll = vessel.ctrlState.roll;
        float yaw = vessel.ctrlState.yaw;
        float throttle = vessel.ctrlState.mainThrottle * Overload;
        float y = -vessel.ctrlState.Y * Overload * 10;
        float x = -vessel.ctrlState.X * Overload * 10;
        float z = throttle * 10 - vessel.ctrlState.Z * Overload * 10;
        float enginecount = 0;
        float tefactor = 1;
        float ID = GetInstanceID();

        foreach (Part p in this.vessel.Parts)
        {
            foreach (PartModule m in p.Modules)
            {
                Taudrive drive = null;
                GNdrive gdrive = null;
                if (m.moduleName == "Taudrive")
                {
                    drive = (Taudrive)m;
                    if (drive.agActivated == true)
                    { 
                        enginecount += 1; 
                    }
                    //if (modified == true && drive.GetInstanceID() != GetInstanceID())
                    //{
                    //    if (drive.modified == true)
                    //    {
                    //        agActivated = drive.agActivated;
                    //        engineIgnited = drive.engineIgnited;
                    //        Overload = drive.Overload;
                    //        overloadtemp = drive.Overload;
                    //
                    //    }
                    //    else
                    //    {
                    //        drive.agActivated = agActivated;
                    //        drive.engineIgnited = engineIgnited;
                    //        drive.Overload = Overload;
                    //        drive.overloadtemp = Overload;
                    //    }
                    //}
                }
                else
                    if (m.moduleName == "GNdrive")
                    {
                        gdrive = (GNdrive)m;
                        if (gdrive.agActivated == true)
                        {
                            enginecount += 1;
                        }
                    }
            }
        }

        if (engineIgnited == true)
        {
            ES = "Activated";
            Events["Deactivate"].guiActive = true;
            Events["Activate"].guiActive = false;
        }
        else
        {
            Events["Deactivate"].guiActive = false;
            Events["Activate"].guiActive = true;
        }

        if (agActivated == true)
        {
            ES = "Activated";
            Events["Deactivateag"].guiActive = true;
            Events["Activateag"].guiActive = false;
        }
        else
        {
            Events["Deactivateag"].guiActive = false;
            Events["Activateag"].guiActive = true;
        }

        if (depleted == true)
        {
            ES = "GNparticle depleted";
            Deactivate();
            Deactivateag();
        }

        Vector3 srfVelocity = vessel.GetSrfVelocity();
        float VerticalV;
        VerticalV = (float)vessel.verticalSpeed;
        Vector3 Airspeed = vessel.transform.InverseTransformDirection(srfVelocity);
        Vector3 gee = FlightGlobals.getGeeForceAtPosition(this.vessel.transform.position) / enginecount;
        Vector3 controlforce = vessel.ReferenceTransform.up * z + vessel.ReferenceTransform.forward * y + vessel.ReferenceTransform.right * x;
        //if (enginecount > maxenginecount)
        //{
        //    ES = "Unsynchronized";
        //   controlforce = Vector3.zero;
        //    gee = Vector3.zero;
        //}

        if (engineIgnited == false)
        {
            controlforce = Vector3.zero;
        }
        if (agActivated == false)
        {
            gee = Vector3.zero;
        }

        float consumption = vessel.GetTotalMass() * Mathf.Abs((-gee + controlforce).magnitude) * fuelefficiency * TimeWarp.deltaTime;
        if (ecActivated == true)
        {
            float elcconsume = particlegrate * ConvertRatio * TimeWarp.deltaTime;
            float elcDrawn = this.part.RequestResource("ElectricCharge", elcconsume);
            float ratio = elcDrawn / elcconsume;
            float GNDrawn = this.part.RequestResource("GNparticle", -(elcconsume / ConvertRatio) * ratio);
            float backcharge = this.part.RequestResource("ElectricCharge",-GNDrawn * ConvertRatio - elcDrawn);
            Debug.Log("elcDrawn" + elcDrawn);
            Debug.Log("GNDrawn" + GNDrawn);
            Debug.Log("backcharge " + backcharge);

        }

        float GNconsumtion = this.part.RequestResource("GNparticle", consumption);
        color = new Vector4(1F, 0F, 36F / 255F, 1F);

        if (consumption !=0 && GNconsumtion <consumption)
        {
            depleted = true;
            controlforce = Vector3.zero;
            gee = Vector3.zero;
            agActivated = false;
            engineIgnited = false;
            Deactivate();
            Deactivateag();
            part.Resources["GNparticle"].amount = 0;

        }

        Debug.Log("Consumption " + consumption);
        Debug.Log("tefactor " + tefactor);
        Debug.Log("Time.deltaTime " + TimeWarp.deltaTime);

        mass = vessel.GetTotalMass().ToString("R");

        if (engineIgnited == true)
        {
            foreach (Part p in this.vessel.parts)
            {
                if ((p.physicalSignificance == Part.PhysicalSignificance.FULL) && (p.rigidbody != null))
                {
                    p.rigidbody.AddForce(controlforce * p.rigidbody.mass);
                }
            }
        }


        if (agActivated == true)
        {
            foreach (Part p in this.vessel.parts)
            {
                if ((p.physicalSignificance == Part.PhysicalSignificance.FULL) && (p.rigidbody != null))
                {
                    p.rigidbody.AddForce(-gee * p.rigidbody.mass);
                }
            }
        }
        emt.minEmission = 10 * (Mathf.Abs(controlforce.magnitude) + 5);
        emt.maxEmission = emt.minEmission;
        rotor.renderer.material.SetColor("_EmissiveColor", color);
        stator.renderer.material.SetColor("_EmissiveColor", color);
        stator.light.color = color;

        rotor.transform.localEulerAngles = new Vector3(0, rotation, 0);
        rotation += 6 * (Mathf.Abs(controlforce.magnitude) + 1) * 120 * TimeWarp.deltaTime;
        while (rotation > 360) rotation -= 360;
        while (rotation < 0) rotation += 360;

    }

}

